@@ -90,9 +90,10 @@ gcm_prefs_combobox_add_profile (GtkWidget *widget,
                                 GcmPrefsEntryType entry_type,
                                 GtkTreeIter *iter)
 {
+  const gchar *id;
   GtkTreeModel *model;
   GtkTreeIter iter_tmp;
-  const gchar *description;
+  GString *string;
 
   /* iter is optional */
   if (iter == NULL)
@@ -102,21 +103,46 @@ gcm_prefs_combobox_add_profile (GtkWidget *widget,
   if (entry_type == GCM_PREFS_ENTRY_TYPE_IMPORT)
     {
       /* TRANSLATORS: this is where the user can click and import a profile */
-      description = _("Other profile…");
+      string = g_string_new (_("Other profile…"));
     }
   else
     {
-      description = cd_profile_get_title (profile);
+      string = g_string_new (cd_profile_get_title (profile));
+
+      /* any source prefix? */
+      id = cd_profile_get_metadata_item (profile,
+                                         CD_PROFILE_METADATA_DATA_SOURCE);
+      if (g_strcmp0 (id, CD_PROFILE_METADATA_DATA_SOURCE_EDID) == 0)
+        {
+          /* TRANSLATORS: this is a profile prefix to signify the
+           * profile has been auto-generated for this hardware */
+          g_string_prepend (string, _("Default: "));
+        }
+#if CD_CHECK_VERSION(0,1,14)
+      if (g_strcmp0 (id, CD_PROFILE_METADATA_DATA_SOURCE_STANDARD) == 0)
+        {
+          /* TRANSLATORS: this is a profile prefix to signify the
+           * profile his a standard space like AdobeRGB */
+          g_string_prepend (string, _("Colorspace: "));
+        }
+      if (g_strcmp0 (id, CD_PROFILE_METADATA_DATA_SOURCE_TEST) == 0)
+        {
+          /* TRANSLATORS: this is a profile prefix to signify the
+           * profile is a test profile */
+          g_string_prepend (string, _("Test profile: "));
+        }
+#endif
     }
 
   /* also add profile */
   model = gtk_combo_box_get_model (GTK_COMBO_BOX(widget));
   gtk_list_store_append (GTK_LIST_STORE(model), iter);
   gtk_list_store_set (GTK_LIST_STORE(model), iter,
-                      GCM_PREFS_COMBO_COLUMN_TEXT, description,
+                      GCM_PREFS_COMBO_COLUMN_TEXT, string->str,
                       GCM_PREFS_COMBO_COLUMN_PROFILE, profile,
                       GCM_PREFS_COMBO_COLUMN_TYPE, entry_type,
                       -1);
+  g_string_free (string, TRUE);
 }
 
 static void
@@ -291,6 +317,9 @@ static gboolean
 gcm_prefs_is_profile_suitable_for_device (CdProfile *profile,
                                           CdDevice *device)
 {
+#if CD_CHECK_VERSION(0,1,14)
+  const gchar *data_source;
+#endif
   CdProfileKind profile_kind_tmp;
   CdProfileKind profile_kind;
   CdColorspace profile_colorspace;
@@ -311,6 +340,14 @@ gcm_prefs_is_profile_suitable_for_device (CdProfile *profile,
   if (profile_kind_tmp != profile_kind)
     goto out;
 
+#if CD_CHECK_VERSION(0,1,14)
+  /* ignore the colorspace profiles */
+  data_source = cd_profile_get_metadata_item (profile,
+                                              CD_PROFILE_METADATA_DATA_SOURCE);
+  if (g_strcmp0 (data_source, CD_PROFILE_METADATA_DATA_SOURCE_STANDARD) == 0)
+    goto out;
+#endif
+
   /* success */
   ret = TRUE;
 out:
@@ -448,6 +485,7 @@ out:
 static void
 gcm_prefs_profile_add_cb (GtkWidget *widget, CcColorPanel *prefs)
 {
+  const gchar *title;
   CdProfile *profile = NULL;
   CcColorPanelPrivate *priv = prefs->priv;
 
@@ -457,6 +495,38 @@ gcm_prefs_profile_add_cb (GtkWidget *widget, CcColorPanel *prefs)
   profile = cd_device_get_default_profile (priv->current_device);
   gcm_prefs_add_profiles_suitable_for_devices (prefs, widget, profile);
 
+  /* set the title */
+  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+                                               "label_assign_title"));
+  switch (cd_device_get_kind (priv->current_device)) {
+    case CD_DEVICE_KIND_DISPLAY:
+      /* TRANSLATORS: this is the dialog title in the 'Add profile' UI */
+      title = _("Available Profiles for Displays");
+      break;
+    case CD_DEVICE_KIND_SCANNER:
+      /* TRANSLATORS: this is the dialog title in the 'Add profile' UI */
+      title = _("Available Profiles for Scanners");
+      break;
+    case CD_DEVICE_KIND_PRINTER:
+      /* TRANSLATORS: this is the dialog title in the 'Add profile' UI */
+      title = _("Available Profiles for Printers");
+      break;
+    case CD_DEVICE_KIND_CAMERA:
+      /* TRANSLATORS: this is the dialog title in the 'Add profile' UI */
+      title = _("Available Profiles for Cameras");
+      break;
+    case CD_DEVICE_KIND_WEBCAM:
+      /* TRANSLATORS: this is the dialog title in the 'Add profile' UI */
+      title = _("Available Profiles for Webcams");
+      break;
+    default:
+      /* TRANSLATORS: this is the dialog title in the 'Add profile' UI
+       * where the device type is not recognised */
+      title = _("Available Profiles");
+      break;
+  }
+  gtk_label_set_label (GTK_LABEL (widget), title);
+
   /* show the dialog */
   widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
                                                "dialog_assign"));
@@ -588,8 +658,7 @@ gcm_prefs_profile_view_cb (GtkWidget *widget, CcColorPanel *prefs)
       g_error_free (error);
     }
 
-  if (argv != NULL)
-    g_ptr_array_unref (argv);
+  g_ptr_array_unref (argv);
   g_free (options);
   if (profile != NULL)
     g_object_unref (profile);
@@ -1089,7 +1158,6 @@ gcm_prefs_set_combo_simple_text (GtkWidget *combo_box)
   g_object_set (renderer,
                 "ellipsize", PANGO_ELLIPSIZE_END,
                 "wrap-mode", PANGO_WRAP_WORD_CHAR,
-                "width-chars", 60,
                 NULL);
   gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo_box), renderer, TRUE);
   gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo_box), renderer,
@@ -1230,36 +1298,28 @@ gcm_prefs_client_sensor_changed_cb (CdClient *client,
 static const gchar *
 gcm_prefs_device_kind_to_icon_name (CdDeviceKind kind)
 {
-  if (kind == CD_DEVICE_KIND_DISPLAY)
+  switch (kind) {
+  case CD_DEVICE_KIND_DISPLAY:
     return "video-display";
-  if (kind == CD_DEVICE_KIND_SCANNER)
+  case CD_DEVICE_KIND_SCANNER:
     return "scanner";
-  if (kind == CD_DEVICE_KIND_PRINTER)
+  case CD_DEVICE_KIND_PRINTER:
     return "printer";
-  if (kind == CD_DEVICE_KIND_CAMERA)
+  case CD_DEVICE_KIND_CAMERA:
     return "camera-photo";
-  if (kind == CD_DEVICE_KIND_CAMERA)
-    return "camera-photo";
-  if (kind == CD_DEVICE_KIND_WEBCAM)
+  case CD_DEVICE_KIND_WEBCAM:
     return "camera-web";
-  return "image-missing";
-}
-
-static gboolean
-gcm_prefs_profile_is_based_from_edid (CdProfile *profile)
-{
-  if (cd_profile_get_kind (profile) != CD_PROFILE_KIND_DISPLAY_DEVICE)
-    return FALSE;
-  if (cd_profile_get_metadata_item (profile, "EDID_md5") == NULL)
-    return FALSE;
-  return TRUE;
+  default:
+    return "image-missing";
+  }
 }
 
 static GString *
 gcm_prefs_get_profile_age_as_string (CdProfile *profile)
 {
+  const gchar *id;
   gint64 age;
-  GString *string;
+  GString *string = NULL;
 
   if (profile == NULL)
     {
@@ -1268,6 +1328,18 @@ gcm_prefs_get_profile_age_as_string (CdProfile *profile)
       goto out;
     }
 
+  /* don't show details for EDID, colorspace or test profiles */
+  id = cd_profile_get_metadata_item (profile,
+                                     CD_PROFILE_METADATA_DATA_SOURCE);
+  if (g_strcmp0 (id, CD_PROFILE_METADATA_DATA_SOURCE_EDID) == 0)
+    goto out;
+#if CD_CHECK_VERSION(0,1,14)
+  if (g_strcmp0 (id, CD_PROFILE_METADATA_DATA_SOURCE_STANDARD) == 0)
+    goto out;
+  if (g_strcmp0 (id, CD_PROFILE_METADATA_DATA_SOURCE_TEST) == 0)
+    goto out;
+#endif
+
   /* days */
   age = cd_profile_get_age (profile);
   if (age == 0)
@@ -1349,6 +1421,8 @@ gcm_prefs_get_profile_title (CcColorPanel *prefs, CdProfile *profile)
 
   g_return_val_if_fail (profile != NULL, NULL);
 
+  string = NULL;
+
   /* get properties */
   ret = cd_profile_connect_sync (profile,
                                  priv->cancellable,
@@ -1447,29 +1521,32 @@ static GtkTreeIter *
 get_iter_for_profile (GtkTreeModel *model, CdProfile *profile, GtkTreeIter *parent)
 {
   const gchar *id;
-  GtkTreeIter iter_tmp;
   gboolean ret;
-  GtkTreeIter *iter = NULL;
+  GtkTreeIter iter;
   CdProfile *profile_tmp;
 
   /* get first element */
-  ret = gtk_tree_model_iter_children (model, &iter_tmp, parent);
+  ret = gtk_tree_model_iter_children (model, &iter, parent);
   if (!ret)
-    goto out;
+    return NULL;
 
   /* remove the other elements */
   id = cd_profile_get_id (profile);
-  do
+  while (ret)
     {
-      gtk_tree_model_get (model, &iter_tmp,
+      gtk_tree_model_get (model, &iter,
               GCM_PREFS_COLUMN_PROFILE, &profile_tmp,
               -1);
       if (g_strcmp0 (id, cd_profile_get_id (profile_tmp)) == 0)
-        iter = &iter_tmp;
+        {
+          g_object_unref (profile_tmp);
+          return gtk_tree_iter_copy (&iter);
+        }
       g_object_unref (profile_tmp);
-    } while (iter == NULL && gtk_tree_model_iter_next (model, &iter_tmp));
-out:
-  return iter;
+      ret = gtk_tree_model_iter_next (model, &iter);
+    }
+
+  return NULL;
 }
 
 static void
@@ -1549,6 +1626,12 @@ gcm_prefs_device_set_model_by_iter (CcColorPanel *prefs, CdDevice *device, GtkTr
 
   /* yay! */
   status = gcm_prefs_get_profile_age_as_string (profile);
+  if (status == NULL)
+    {
+      status = g_string_new (_("Uncalibrated"));
+      g_string_prepend (status, "<span foreground='gray'><i>");
+      g_string_append (status, "</i></span>");
+    }
 
   /* greater than the calibration threshold for the device type */
   age = cd_profile_get_age (profile);
@@ -1592,8 +1675,9 @@ skip:
       profile_tmp = g_ptr_array_index (profiles, i);
       title_tmp = gcm_prefs_get_profile_title (prefs, profile_tmp);
 
-      /* don't show details for EDID profiles */
-      if (gcm_prefs_profile_is_based_from_edid (profile_tmp))
+      /* get profile age */
+      date_tmp = gcm_prefs_get_profile_age_as_string (profile_tmp);
+      if (date_tmp == NULL)
         {
           /* TRANSLATORS: this is when the calibration profile age is not
            * specified as it has been autogenerated from the hardware */
@@ -1601,22 +1685,15 @@ skip:
           g_string_prepend (date_tmp, "<span foreground='gray'><i>");
           g_string_append (date_tmp, "</i></span>");
         }
-      else
-        {
-          date_tmp = gcm_prefs_get_profile_age_as_string (profile_tmp);
-        }
       sort_tmp = gcm_prefs_get_profile_created_for_sort (profile_tmp);
 
       /* get an existing profile, or create a new one */
       iter_tmp_p = get_iter_for_profile (GTK_TREE_MODEL (priv->list_store_devices),
                                          profile_tmp, iter);
       if (iter_tmp_p == NULL)
-        {
-          gtk_tree_store_append (priv->list_store_devices, &iter_tmp, iter);
-          iter_tmp_p = &iter_tmp;
-        }
+        gtk_tree_store_append (priv->list_store_devices, &iter_tmp, iter);
 
-      gtk_tree_store_set (priv->list_store_devices, iter_tmp_p,
+      gtk_tree_store_set (priv->list_store_devices, iter_tmp_p ? iter_tmp_p : &iter_tmp,
                           GCM_PREFS_COLUMN_DEVICE, device,
                           GCM_PREFS_COLUMN_PROFILE, profile_tmp,
                           GCM_PREFS_COLUMN_DEVICE_PATH, cd_device_get_object_path (device),
@@ -1626,6 +1703,8 @@ skip:
                           GCM_PREFS_COLUMN_RADIO_VISIBLE, TRUE,
                           GCM_PREFS_COLUMN_RADIO_ACTIVE, i==0,
                           -1);
+      if (iter_tmp_p != NULL)
+        gtk_tree_iter_free (iter_tmp_p);
       g_free (title_tmp);
       g_free (sort_tmp);
       g_string_free (date_tmp, TRUE);
